/* File : CBT.cxx */

#include "CBT.h"


int 
Ace::size()
{
  return(mA.size());
}

int 
Ace::ContigNum()
{
  return(mA.size());
}

void
Ace::Close()
{
  mA.Close();
}

int
Ace::ReadContigBatch()
{
  int rValue = mA.ReadContigBatch();
  return(rValue);
}

int 
Ace::GetReadQuality(char* cReadQualName)
{
	 string ReadQualName = cReadQualName;

	 // Now I need a hash that maps read names to the contig and read number
	 // within that contig.  
	 mA.BuildReadHash();

	 cerr<<"Reading "<<ReadQualName<<"..."<<flush;

	 if (Contains(ReadQualName,".gz")){
		  izipstream zin;
		  zin.open(ReadQualName.c_str());
		  mA.GetReadsQuality(zin);
		  zin.close();
	 }else{
		  ifstream fin;
		  fin.open(ReadQualName.c_str());
		  mA.GetReadsQuality(fin);
		  fin.close();
	 }
	 cerr<<"done."<<endl;
	 
	 mA.CopyBQToQuality();
	 
	 // Goes through all of the reads and makes sure that they have some 
	 // defined quality values, zeros are added where currently undefined. 
	 // The purpose is to ensure that other code that expects the quality 
	 // values to be there won't crash trying to access nonexistent values
	 // (and it's easier to do this once than to test each instance to see if 
  // the quality exists). 
	 mA.AddFakeQualityValues();
	 
	 mA.CalculateReadTrims(1);
}

int  
Ace::Open(char* AceName)
{
  string Name = AceName;
  int rValue = mA.Open(Name);
  return(rValue);
}

int 
Ace::GetNumReads(int cNum)
{
  return(mA[cNum].mReads.size());
}

int 
Ace::GetContigSize(int cNum){
  return(mA[cNum].size());
}

char 
Ace::GetAlignedCharacter(int cNum,int readNum,int cIdx)
{
  Contig &C = mA[cNum];
  return(C.GetAlignedCharacter(readNum,cIdx));
}

const char *
Ace::GetAlignedCharacters(int cNum,int windowStart,int windowEnd,int cIdx)
{
  string temp = mA[cNum].GetAlignedCharacters(windowStart,windowEnd,cIdx);
  return(temp.c_str());
}

const char* 
Ace::GetContigName(int cNum)
{
	 return(mA[cNum].mName.c_str());
}

void 
Ace::ComputeTemplateCoverage(int cNum){
  if (cNum < mA.size()){
	 mA[cNum].GetTemplateCoverage(mTCover);
  }
}

void 
Ace::ComputeBadTemplateCoverage(int cNum){
  if (cNum < mA.size()){
	 mA[cNum].GetBadTemplateCoverage(mBadTCover);
  }
}

void
Ace::ComputeBadTemplateEndPoints(int cNum){

  mRStart1.clear();
  mRStart2.clear();
  mREnd1.clear();
  mREnd2.clear();

  if (cNum < mA.size()){
	 vector<TemplateInfo> & TInfo = mA[cNum].mTInfo;
	 for(int i = 0;i< TInfo.size();i++){		
		TemplateInfo &T = TInfo[i];

		int start1,end1;
		int start2,end2;

		if (T.mbPair && (!T.mbOrientationOK)){

		  cout<<cNum<<"\t"<<T.mRName1<<"\t"<<T.mRDir1<<"\t"<<T.mRStart1<<"\t"<<T.mRSize1;
		  cout<<"\t"<<T.mRName2<<"\t"<<T.mRDir2<<"\t"<<T.mRStart2<<"\t"<<T.mRSize2<<endl;

		  if (T.mRDir1 == 'U'){
			 start1 = T.mRStart1;
			 end1 = T.mRStart1+T.mRSize1-1;
		  }else{
			 start1 = T.mRStart1+T.mRSize1-1;
			 end1 = T.mRStart1;
		  }
		  mRStart1.push_back(start1);
		  mREnd1.push_back(end1);

		  if (T.mRDir2 == 'U'){
			 start2 = T.mRStart2;
			 end2 = T.mRStart2+T.mRSize2-1;
		  }else{
			 start2 = T.mRStart2+T.mRSize2-1;
			 end2 = T.mRStart2;
		  }
		  mRStart2.push_back(start2);
		  mREnd2.push_back(end2);
		}
	 }
  }
}

int 
Ace::GetNumBadTemplates(){
  return(mRStart1.size());
}

int 
Ace::GetRStart1(int tIdx){
  return(mRStart1[tIdx]);
}

int 
Ace::GetRStart2(int tIdx){
  return(mRStart2[tIdx]);
}
int  
Ace::GetREnd1(int tIdx){
  return(mREnd1[tIdx]);
}
int  
Ace::GetREnd2(int tIdx){
  return(mREnd2[tIdx]);
}


void 
Ace::ComputeTemplateSpan(int cNum){
  if (cNum < mA.size()){
	 mA[cNum].GetTemplateSpan(mTSpan);
  }
}

int 
Ace::GetTemplateCoverage(int cIdx){
  if (cIdx < mTCover.size()){
	 return(mTCover[cIdx]);
  }else return(-1);
}

int 
Ace::GetBadTemplateCoverage(int cIdx){
  if (cIdx < mBadTCover.size()){
	 return(mBadTCover[cIdx]);
  }else return(-1);
}


int 
Ace::GetTemplateSpan(int cIdx){
  if (cIdx < mTSpan.size()){
	 return(mTSpan[cIdx]);
  }else return(-1);
}

void 
Ace::ComputeCoverage(int cNum){
  if (cNum < mA.size()){
	 mA[cNum].GetCoverage(mCover);
  }
}

int 
Ace::GetCoverage(int cIdx){
  if (cIdx < mCover.size()){
	 return(mCover[cIdx]);
  }else return(-1);
}

void 
Ace::ComputeWGSBACCoverage(int cNum){
  if (cNum < mA.size()){
	 mA[cNum].GetWGSBACCoverage(mWGSCover,mBACCover);
  }
}

int 
Ace::GetBACCoverage(int cIdx){
  if (cIdx < mBACCover.size()){
	 return(mBACCover[cIdx]);
  }else return(-1);
}


int 
Ace::GetWGSCoverage(int cIdx){
  if (cIdx < mWGSCover.size()){
	 return(mWGSCover[cIdx]);
  }else return(-1);
}

void 
Ace::ComputeQuality(int cNum){
  mQuality.clear();
  if (cNum < mA.size()){
	 Contig &C = mA[cNum];
	 for(int i = 0;i< C.size();i++){
		mQuality.push_back(C.mConsensusQuality[i]);
	 }
  }
}

int 
Ace::GetQuality(int cIdx){
  if (cIdx < mQuality.size()){
	 return(mQuality[cIdx]);
  }else return(-1);
}


void
Ace::ComputeHQDescrepency(int cNum){
  mHQDesc.clear();
  if (cNum < mA.size()){
	 Contig &C = mA[cNum];
	 C.GetHQDescrepency(mHQDesc);
  }
}

int
Ace::GetHQDescrepency(int cIdx){
  if (cIdx < mHQDesc.size()){
	 return(mHQDesc[cIdx]);
  }else return(-1);
}
